<?php

App::uses('CakeEmail', 'Network/Email');
App::uses('AuthComponent', 'Controller/Component');


/**
 * Business logic for user accounts
 */
class User extends AppModel {

	public $displayField = "email";

	public $order = array('email' => 'ASC');

	public $belongsTo = array(
		'UserGroup'
	);

	const DEFAULT_USR_GROUP = 3;

/**
 * Ensure the password is encrypted if present before it's saved to the DB
 *
 * @param array $options beforeSave options
 * @return void
 */
	public function beforeSave($options = array()) {

		if (isset($this->data[$this->alias]['password']) && ! empty($this->data[$this->alias]['password'])) {

			$this->data[$this->alias]['password'] = AuthComponent::password($this->data[$this->alias]['password']);
		} else {

			unset($this->data[$this->alias]['password']);
		}

		return;

	}


	public $validate = array(
		'user_group_id' => array(
			'required' => array(
				'rule' => 'notEmpty',
				'message' => 'User group cannot be blank'
			)
		),
		'email' => array(
			'required' => array(
				'rule' => 'notEmpty',
				'message' => 'Email cannot be left blank'
			),
			'email' => array(
				'rule' => 'email',
				'message' => 'You must enter a valid email address'
			),
			'unique' => array(
				'rule' => 'isUnique',
				'message' => 'A user with that email address already exists'
			)
		),
		'password' => array(
			'required' => array(
				'rule' => 'notEmpty',
				'message' => 'Password cannot be left blank',
				'on' => 'create'
			)
		),
		'confirm_password' => array(
			'required' => array(
				'rule' => 'notEmpty',
				'message' => 'You must confirm your password',
				'on' => 'create'
			),
			'matches' => array(
				'rule' => array('checkMatches', 'password'),
				'message' => 'The passwords do not match'
			)
		)

	);


/**
 * Validate password rules
 */
	public function validatePassword() {

		return array(
			'password' => array(
				'required' => array(
					'rule' => 'notEmpty',
					'message' => 'Password cannot be left blank'
				)
			),
			'confirm_password' => array(
				'required' => array(
					'rule' => 'notEmpty',
					'message' => 'You must confirm your password'
				),
				'matches' => array(
					'rule' => array('checkMatches', 'password'),
					'message' => 'The passwords do not match'
				)
			)
		);

	}


	public function validateEdit() {

		$rules = $this->validate;

		unset($rules['password']);
		unset($rules['confirm_password']);

		return $rules;

	}


/**
 * Validation rules for a login form submission
 *
 * @return array
 */
	public function validateLogin() {

		return array(
			'email' => array(
				'required' => array(
					'rule' => 'notEmpty',
					'message' => 'Email cannot be left blank.'
				),
				'email' => array(
					'rule' => 'email',
					'message' => 'You must enter a valid email address'
				)
			),
			'password' => array(
				'required' => array(
					'rule' => 'notEmpty',
					'message' => 'Password cannot be left blank'
				)
			)
		);

	}


/**
 * Returns true if an email already exists
 *
 * @param array $check email field to check
 * @return true if email doesn't exists, otherwise false
 */
	public function checkIsRegistered($check) {

		App::uses('User', 'Model');

		$value = array_pop($check);

		$user = $this->User->findByEmail($value);

		return empty($user);

	}

/**
 * Gets the user and all associated data out that is needed for login
 * @param  int $id  The ID of the user
 * @return array 	User array
 */
	public function getUserForLogin($id) {

		$fullUser = $this->find('first', array(
				'conditions' => array(
					$this->alias.'.id' => $id
				),
				'contain'=> array(
					'UserGroup'
				)
			)
		);

		return $fullUser;

	}


/**
 * Sets a password reset code for a user.
 *
 * @param integer $userId
 * @return boolean - returns false if the user cannot be found
 */
	public function resetPassword($userId) {

		$user = $this->findById($userId);

		if (!empty($user)) {
			$code = $user['User']['password_reset_code'];

			if (empty($code)) {
				$code = substr(md5(uniqid()), 0, 10);
				$this->id = $userId;
				$this->saveField('password_reset_code', $code);
			}

			App::uses('EvUserPasswordResetListener','EvCore.Lib/Event');
			$this->getEventManager()->attach(new EvUserPasswordResetListener());
			$event = new CakeEvent('Model.User.passwordReset', $this, array(
					'Model' => $this->alias,
					'User' => $user,
					'code' => $code,
					'url' => Router::url(
						array(
							'plugin' => false,
							'controller' => 'ev_core_users',
							'action' => 'password_reset_callback',
							$code
						),
						true
					)
				)
			);
			$this->getEventManager()->dispatch($event);

			return true;
		}

		return false;
	}

	/**
	 * register a new user
	 *
	 * @param 	array 	data from request->data
	 * @return  bool
	 */
	public function register($data)
	{
		$activationType = Configure::read('SiteSetting.users.activation');

		$data[$this->alias]['is_active'] = 0;
		if ($activationType === 'auto') {
			$data[$this->alias]['is_active'] = 1;
		}

		$data[$this->alias]['user_group_id'] = self::DEFAULT_USR_GROUP;
		$return = $this->saveAll($data);

		if ($return) {

            // dispatch new user event
            $this->getEventManager()->dispatch(
                new CakeEvent('Model.User.newUser', $this, array(
                    'Model' => $this->alias,
                    'user' => $data
                ))
            );

			if ($activationType == 'auto') {

				// dispatch auto activate event
				$this->getEventManager()->dispatch(
					new CakeEvent('Model.User.autoActivate', $this, array(
						'Model' => $this->alias,
					    'User' => $data
					))
				);

			} elseif ($activationType == 'email') {

				$code = hash('sha1', $this->data[$this->alias]['id'] . '-' . $this->data[$this->alias]['email'] . '-' . time());
				$this->saveField('verification_code', $code);

				// dispatch email activate event
				$this->getEventManager()->dispatch(
					new CakeEvent('Model.User.emailActivate', $this, array(
						'Model' => $this->alias,
					    'User' => $data,
						'url' => Router::url(
							array(
								'plugin' => false,
								'controller' => 'ev_core_users',
								'action' => 'verify_email',
								$code
							),
							true
						)
					))
				);
			}
		}

		return $return;
	}
}